@@ -36,14 +36,13 @@ SEXP fastmean(SEXP args)
   if (length(args)>2) {
     tmp = CADDR(args);
     if (!isLogical(tmp) || LENGTH(tmp)!=1 || LOGICAL(tmp)[0]==NA_LOGICAL)
-      error("narm should be TRUE or FALSE");
+      error(_("narm should be TRUE or FALSE"));  // # nocov ; [.data.table should construct the .External call correctly
     narm=LOGICAL(tmp)[0];
   }
   PROTECT(ans = allocNAVector(REALSXP, 1));
+  copyMostAttrib(x, ans);
   if (!isInteger(x) && !isReal(x) && !isLogical(x)) {
-    warning("argument is not numeric or logical: returning NA");
-    UNPROTECT(1);
-    return(ans);
+    error(_("fastmean was passed type %s, not numeric or logical"), type2char(TYPEOF(x)));
   }
   l = LENGTH(x);
   if (narm) {
@@ -81,7 +80,7 @@ SEXP fastmean(SEXP args)
       REAL(ans)[0] = (double) s;
       break;
     default:
-      error("Type '%s' not supported in fastmean", type2char(TYPEOF(x)));
+      error(_("Internal error: type '%s' not caught earlier in fastmean"), type2char(TYPEOF(x)));  // # nocov
     }
   } else {  // narm==FALSE
     switch(TYPEOF(x)) {
@@ -109,10 +108,9 @@ SEXP fastmean(SEXP args)
       REAL(ans)[0] = (double) s;
       break;
     default:
-      error("Type '%s' not supported in fastmean", type2char(TYPEOF(x)));
+      error(_("Internal error: type '%s' not caught earlier in fastmean"), type2char(TYPEOF(x)));  // # nocov
     }
   }
-  copyMostAttrib(x, ans);
   UNPROTECT(1);
   return(ans);
 }
@@ -137,5 +135,3 @@ SEXP fastmean(SEXP args)
       break;
 */
 
-
-
